package collector

import (
	"context"
	"strings"

	arvancloud "git.arvancloud.ir/arvancloud/cdn-go-sdk"
	"github.com/prometheus/client_golang/prometheus"
	log "github.com/sirupsen/logrus"
)

type cdnCollector struct {
	props        []string
	descriptions map[string]*prometheus.Desc
	period       string
	apiClient    *arvancloud.APIClient
	domain       string
}

var (
	mainCtx = context.Background()
)

func newCDNCollector(domain string) ArvancloudCollector {
	c := &cdnCollector{
		domain: domain,
	}
	c.init()
	return c
}

func (c *cdnCollector) init() {
	const prefix = "cdn"

	log.Debugf("Initializing CDN collector for domain: %s", c.domain)

	c.descriptions = make(map[string]*prometheus.Desc)

	c.descriptions["traffic"] = description(prefix, "traffic", "CDN domain's traffic", []string{"domain", "cache_type"})
	c.descriptions["request"] = description(prefix, "requests", "CDN domain's request", []string{"domain", "cache_type"})
	c.descriptions["visitors"] = description(prefix, "visitors", "CDN domain's unique visitors", []string{"domain"})
	c.descriptions["high_request_ips"] = description(prefix, "high_request_ips", "CDN domain's top ips with highest requests", []string{"domain", "ip"})
	c.descriptions["response_time"] = description(prefix, "response_time", "CDN domain's response time", []string{"domain"})
	c.descriptions["requests_by_status"] = description(prefix, "requests_by_status", "CDN domain's requests by status code", []string{"domain", "status"})
	c.descriptions["traffic_by_country"] = description(prefix, "traffic_by_country", "CDN domain's traffic counter by country", []string{"domain", "country_code", "country_name"})
	c.descriptions["requests_by_country"] = description(prefix, "requests_by_country", "CDN domain's request counter by country", []string{"domain", "country_code", "country_name"})
}

func (c *cdnCollector) describe(ch chan<- *prometheus.Desc) {
	for _, d := range c.descriptions {
		ch <- d
	}
}

func (c *cdnCollector) collect(ctx *collectorContext) error {
	log.Debugf("Collecting CDN metrics for domain: %s", c.domain)

	configuration := arvancloud.NewConfiguration()
	configuration.AddDefaultHeader("authorization", ctx.cfg.Token)
	c.apiClient = arvancloud.NewAPIClient(configuration)

	if err := c.getTrafficReport(ctx, c.domain); err != nil {
		log.Errorf("Error when getting `Reports Traffics Total`: %v\n", err)
		return err
	}

	if err := c.getVisitorReport(ctx, c.domain); err != nil {
		log.Errorf("Error when getting `Reports Visitor`: %v\n", err)
		return err
	}

	if err := c.getHighRequestIPsReport(ctx, c.domain); err != nil {
		log.Errorf("Error when getting `Reports High Requested IPs`: %v\n", err)
		return err
	}

	if err := c.getTrafficGeoMapReport(ctx, c.domain); err != nil {
		log.Errorf("Error when getting `Reports Geo Map Traffic/Requests`: %v\n", err)
		return err
	}

	if err := c.getResponseTimeReport(ctx, c.domain); err != nil {
		log.Errorf("Error when getting `Reports Response Time`: %v\n", err)
		return err
	}

	if err := c.getStatusCodeReport(ctx, c.domain); err != nil {
		log.Errorf("Error when getting `Reports Status Code`: %v\n", err)
		return err
	}

	return nil
}

func (c *cdnCollector) getTrafficReport(ctx *collectorContext, domain string) error {
	trafficTotal, r, err := c.apiClient.ReportsApi.ReportsTrafficsTotal(mainCtx, domain).Period(ctx.cfg.Products.CDN.Period).Execute()
	if err != nil {
		log.Debugf("Full HTTP response `Reports Traffics Total`: %v\n", r)
		return err
	}

	traffics := trafficTotal.Data.Statistics.Traffics
	requests := trafficTotal.Data.Statistics.Requests

	desc := c.descriptions["traffic"]
	ctx.ch <- prometheus.MustNewConstMetric(desc, prometheus.GaugeValue, float64(traffics.GetTotal()), domain, "total")
	ctx.ch <- prometheus.MustNewConstMetric(desc, prometheus.GaugeValue, float64(traffics.GetSaved()), domain, "saved")
	ctx.ch <- prometheus.MustNewConstMetric(desc, prometheus.GaugeValue, float64(traffics.GetBypass()), domain, "bypass")

	desc = c.descriptions["request"]
	ctx.ch <- prometheus.MustNewConstMetric(desc, prometheus.GaugeValue, float64(requests.GetTotal()), domain, "total")
	ctx.ch <- prometheus.MustNewConstMetric(desc, prometheus.GaugeValue, float64(requests.GetSaved()), domain, "saved")
	ctx.ch <- prometheus.MustNewConstMetric(desc, prometheus.GaugeValue, float64(requests.GetBypass()), domain, "bypass")

	return nil
}

func (c *cdnCollector) getVisitorReport(ctx *collectorContext, domain string) error {
	visitors, r, err := c.apiClient.ReportsApi.ReportsVisitorsIndex(mainCtx, domain).Period(ctx.cfg.Products.CDN.Period).Execute()
	if err != nil {
		log.Debugf("Full HTTP response `Reports Visitor`: %v\n", r)
		return err
	}

	desc := c.descriptions["visitors"]
	ctx.ch <- prometheus.MustNewConstMetric(desc, prometheus.GaugeValue, float64(visitors.GetData().Statistics.Visitors.TotalVisitors), domain)

	return nil
}

func (c *cdnCollector) getHighRequestIPsReport(ctx *collectorContext, domain string) error {
	highRequestIps, r, err := c.apiClient.ReportsApi.ReportsVisitorsHighRequestIps(mainCtx, domain).Period(ctx.cfg.Products.CDN.Period).Execute()
	if err != nil && err.Error() != "422 Unprocessable Entity" {
		log.Debugf("Full HTTP response `Reports High Requested IPs`: %v\n", r)
		return err
	}

	desc := c.descriptions["high_request_ips"]

	for _, ip := range highRequestIps.GetData() {
		x, _ := ip.Ip.MarshalJSON()
		ctx.ch <- prometheus.MustNewConstMetric(desc, prometheus.GaugeValue, float64(ip.GetRequestCount()), domain, strings.ReplaceAll(string(x), "\"", ""))
	}

	return nil
}

func (c *cdnCollector) getTrafficGeoMapReport(ctx *collectorContext, domain string) error {
	trafficGeoMap, r, err := c.apiClient.ReportsApi.ReportsTrafficsMap(mainCtx, domain).Period(ctx.cfg.Products.CDN.Period).Execute()
	if err != nil {
		log.Debugf("Full HTTP response `Reports Geo Map Traffic/Requests`: %v\n", r)
		return err
	}

	traffics := trafficGeoMap.Data.Charts.GetTraffics()
	requests := trafficGeoMap.Data.Charts.GetRequests()

	desc := c.descriptions["traffic_by_country"]

	for i, d := range traffics {
		ctx.ch <- prometheus.MustNewConstMetric(desc, prometheus.GaugeValue, float64(d.GetValue()), domain, i, d.GetName())
	}

	desc = c.descriptions["requests_by_country"]

	for i, d := range requests {
		ctx.ch <- prometheus.MustNewConstMetric(desc, prometheus.GaugeValue, float64(d.GetValue()), domain, i, d.GetName())
	}

	return nil
}

func (c *cdnCollector) getResponseTimeReport(ctx *collectorContext, domain string) error {
	responseTime, r, err := c.apiClient.ReportsApi.ReportsResponseTimeIndex(mainCtx, domain).Period(ctx.cfg.Products.CDN.Period).Execute()
	if err != nil {
		log.Debugf("Full HTTP response `Reports Response Time`: %v\n", r)
		return err
	}

	responseCharts := responseTime.GetData().Charts.Ir.Series

	desc := c.descriptions["response_time"]
	ctx.ch <- prometheus.MustNewConstMetric(desc, prometheus.GaugeValue, avgOfArray(responseCharts[0].Data), domain)

	return nil
}

func (c *cdnCollector) getStatusCodeReport(ctx *collectorContext, domain string) error {
	statusCode, r, err := c.apiClient.ReportsApi.ReportsStatusIndex(mainCtx, domain).Period(ctx.cfg.Products.CDN.Period).Execute()
	if err != nil {
		log.Debugf("Full HTTP response `Reports Status Code`: %v\n", r)
		return err
	}

	desc := c.descriptions["requests_by_status"]
	ctx.ch <- prometheus.MustNewConstMetric(desc, prometheus.GaugeValue, float64(statusCode.GetData().Statistics.StatusCodes.GetVar2xxSum()), domain, "2xx")
	ctx.ch <- prometheus.MustNewConstMetric(desc, prometheus.GaugeValue, float64(statusCode.GetData().Statistics.StatusCodes.GetVar3xxSum()), domain, "3xx")
	ctx.ch <- prometheus.MustNewConstMetric(desc, prometheus.GaugeValue, float64(statusCode.GetData().Statistics.StatusCodes.GetVar4xxSum()), domain, "4xx")
	ctx.ch <- prometheus.MustNewConstMetric(desc, prometheus.GaugeValue, float64(statusCode.GetData().Statistics.StatusCodes.GetVar5xxSum()), domain, "5xx")

	return nil
}
