package collector

import (
	"encoding/json"
	"fmt"
	"net/http"
	"time"

	"github.com/prometheus/client_golang/prometheus"
	log "github.com/sirupsen/logrus"
)

const objectAPIHost = "https://panel.arvanstorage.ir"

type objectCollector struct {
	descriptions map[string]*prometheus.Desc
	bucket       string
}

func (c *objectCollector) init() {
	const prefix = "storage"

	log.Debugf("Initializing object collector for bucket: %s", c.bucket)

	c.descriptions = make(map[string]*prometheus.Desc)
	c.descriptions["modified_time"] = description(prefix, "modified_time", "The last timestamp of bucket usage being updated", []string{"bucket"})
	c.descriptions["total_object_count"] = description(prefix, "total_object_count", "Total count of objects in the bucket", []string{"bucket"})
	c.descriptions["multi_part_object_count"] = description(prefix, "multi_part_object_count", "Total count of multi-part objects in the bucket", []string{"bucket"})
	c.descriptions["bucket_size"] = description(prefix, "bucket_size", "Bucket size in bytes", []string{"bucket"})
	c.descriptions["bucket_download"] = description(prefix, "bucket_download", "Total bucket download of today", []string{"bucket"})
	c.descriptions["bucket_upload"] = description(prefix, "bucket_upload", "Total bucket upload of today", []string{"bucket"})
	c.descriptions["request_count"] = description(prefix, "request_count", "Total count of successful operations per type", []string{"bucket", "operation_type"})
}

func newObjectCollector(bucket string) ArvancloudCollector {
	c := &objectCollector{
		bucket: bucket,
	}
	c.init()
	return c
}

func (c *objectCollector) describe(ch chan<- *prometheus.Desc) {
	for _, d := range c.descriptions {
		ch <- d
	}
}

func (c *objectCollector) collect(ctx *collectorContext) error {
	err := c.GetBucketMetrics(ctx)
	if err != nil {
		log.Errorf("Error when getting bucket usage metrics: %v\n", err)
		return err
	}

	return nil
}

// Bucket usage metrics response model
type bucketMetricsResponse struct {
	Message string        `json:"message"`
	Data    bucketMetrics `json:"data"`
}

// Bucket metrics
type bucketMetrics struct {
	MTime                time.Time            `json:"arvancloud_aos_modified_time" example:"2023-11-07T06:18:22.860248911Z"` // Indicates the most recent update time of the data
	TotalObjectCount     int64                `json:"arvancloud_aos_total_object_count" example:"100"`                       // Represents the total count of objects in bucket
	MultiPartObjectCount int64                `json:"arvancloud_aos_multi_part_object_count" example:"5"`                    // Represents the total count of multipart objects
	BucketSize           int64                `json:"arvancloud_aos_bucket_size" example:"52428800"`                         // Represents the actual size of the bucket in bytes
	BucketDownload       int64                `json:"arvancloud_aos_bucket_download" example:"13687300"`                     // Represents bucket downloads in bytes
	BucketUpload         int64                `json:"arvancloud_aos_bucket_upload" example:"1642400"`                        // Represents bucket uploads in bytes
	RequestsByType       bucketRequestsByType `json:"arvancloud_aos_requests_by_type"`                                       // Represents the total count of successful operations
}

// Total count of RGW successful operations per type of operations
type bucketRequestsByType struct {
	PutObject    int64 `json:"put_object" example:"5"`    // Represents the count of successful "put_object" operations
	GetObject    int64 `json:"get_object" example:"43"`   // Represents the count of successful "get_object" operations
	DeleteObject int64 `json:"delete_object" example:"2"` // Represents the count of successful "delete_object"" operations
	PostObject   int64 `json:"post_object" example:"1"`   // Represents the count of successful "post_object" operations
	HeadBucket   int64 `json:"head_bucket" example:"7"`   // Represents the count of successful "head_bucket" operations
}

// Gets bucket usage metrics from object API
func (c *objectCollector) GetBucketMetrics(ctx *collectorContext) error {
	var bucketMetricsResp bucketMetricsResponse
	url := fmt.Sprintf("%v/panel/v1/bucket/%v/usage", objectAPIHost, c.bucket)

	req, err := http.NewRequest("GET", url, nil)
	if err != nil {
		log.Errorf("Error creating request: %v\n", err)
		return err
	}

	req.Header.Set("Authorization", ctx.cfg.Token)

	client := &http.Client{}
	resp, err := client.Do(req)
	if err != nil {
		log.Errorf("Error sending request: %v\n", err)
		return err
	}
	defer resp.Body.Close()

	err = json.NewDecoder(resp.Body).Decode(&bucketMetricsResp)
	if err != nil {
		log.Errorf("Error decoding response: %v\n", err)
		return err
	}

	if resp.StatusCode != 200 {
		return fmt.Errorf("Bucket usage request status code: %v. Message: %v", resp.Status, bucketMetricsResp.Message)
	}

	bucketMetrics := bucketMetricsResp.Data

	desc := c.descriptions["modified_time"]
	ctx.ch <- prometheus.MustNewConstMetric(desc, prometheus.GaugeValue, float64(bucketMetrics.MTime.Unix()), c.bucket)

	desc = c.descriptions["total_object_count"]
	ctx.ch <- prometheus.MustNewConstMetric(desc, prometheus.GaugeValue, float64(bucketMetrics.TotalObjectCount), c.bucket)

	desc = c.descriptions["multi_part_object_count"]
	ctx.ch <- prometheus.MustNewConstMetric(desc, prometheus.GaugeValue, float64(bucketMetrics.MultiPartObjectCount), c.bucket)

	desc = c.descriptions["bucket_size"]
	ctx.ch <- prometheus.MustNewConstMetric(desc, prometheus.GaugeValue, float64(bucketMetrics.BucketSize), c.bucket)

	desc = c.descriptions["bucket_download"]
	ctx.ch <- prometheus.MustNewConstMetric(desc, prometheus.GaugeValue, float64(bucketMetrics.BucketDownload), c.bucket)

	desc = c.descriptions["bucket_upload"]
	ctx.ch <- prometheus.MustNewConstMetric(desc, prometheus.GaugeValue, float64(bucketMetrics.BucketUpload), c.bucket)

	desc = c.descriptions["request_count"]
	ctx.ch <- prometheus.MustNewConstMetric(desc, prometheus.GaugeValue, float64(bucketMetrics.RequestsByType.PutObject), c.bucket, "put_obj")
	ctx.ch <- prometheus.MustNewConstMetric(desc, prometheus.GaugeValue, float64(bucketMetrics.RequestsByType.GetObject), c.bucket, "get_obj")
	ctx.ch <- prometheus.MustNewConstMetric(desc, prometheus.GaugeValue, float64(bucketMetrics.RequestsByType.DeleteObject), c.bucket, "delete_obj")
	ctx.ch <- prometheus.MustNewConstMetric(desc, prometheus.GaugeValue, float64(bucketMetrics.RequestsByType.PostObject), c.bucket, "post_obj")
	ctx.ch <- prometheus.MustNewConstMetric(desc, prometheus.GaugeValue, float64(bucketMetrics.RequestsByType.HeadBucket), c.bucket, "head_bucket")

	return nil
}
