package config

import (
	"bytes"
	"testing"
)

func TestLoadConfig(t *testing.T) {
	yamlConfig := []byte(`
token: "your_token"
products:
  cdn:
    enable: true
    domains:
      - "example.com"
    period: "3h"
  object:
    enable: false
    period: "3h"
`)

	r := bytes.NewReader(yamlConfig)
	config, err := Load(r)

	if err != nil {
		t.Fatalf("Load returned an error: %v", err)
	}

	if config.Token != "your_token" {
		t.Errorf("Expected Token to be 'your_token', but got: %s", config.Token)
	}

	if !config.Products.CDN.Enabled {
		t.Errorf("CDN should be enabled")
	}

	if config.Products.CDN.Period != "3h" {
		t.Errorf("Expected CDN period to be '3h', but got: %s", config.Products.CDN.Period)
	}

	if config.Products.OBJECT.Enabled {
		t.Errorf("OBJECT should not be enabled")
	}
}

func TestEnabledProducts(t *testing.T) {
	config := &Config{
		Token: "test_token",
		Products: struct {
			CDN struct {
				Enabled bool     `yaml:"enable,omitempty"`
				Domains []string `yaml:"domains,omitempty"`
				Period  string   `yaml:"period,omitempty"`
			} `yaml:"cdn,omitempty"`
			OBJECT struct {
				Enabled bool     `yaml:"enable,omitempty"`
				Buckets []string `yaml:"buckets,omitempty"`
			} `yaml:"object,omitempty"`
		}{
			CDN: struct {
				Enabled bool     `yaml:"enable,omitempty"`
				Domains []string `yaml:"domains,omitempty"`
				Period  string   `yaml:"period,omitempty"`
			}{
				Enabled: true,
				Domains: []string{"example.com"},
				Period:  "daily",
			},
			OBJECT: struct {
				Enabled bool     `yaml:"enable,omitempty"`
				Buckets []string `yaml:"buckets,omitempty"`
			}{
				Enabled: false,
			},
		},
	}

	enabledProducts := config.EnabledProducts()

	expected := "CDN"
	if enabledProducts != expected {
		t.Errorf("Expected enabled products to be '%s', but got: %s", expected, enabledProducts)
	}

	config = &Config{
		Token: "test_token",
		Products: struct {
			CDN struct {
				Enabled bool     `yaml:"enable,omitempty"`
				Domains []string `yaml:"domains,omitempty"`
				Period  string   `yaml:"period,omitempty"`
			} `yaml:"cdn,omitempty"`
			OBJECT struct {
				Enabled bool     `yaml:"enable,omitempty"`
				Buckets []string `yaml:"buckets,omitempty"`
			} `yaml:"object,omitempty"`
		}{
			CDN: struct {
				Enabled bool     `yaml:"enable,omitempty"`
				Domains []string `yaml:"domains,omitempty"`
				Period  string   `yaml:"period,omitempty"`
			}{
				Enabled: false,
				Domains: []string{"example.com"},
				Period:  "daily",
			},
			OBJECT: struct {
				Enabled bool     `yaml:"enable,omitempty"`
				Buckets []string `yaml:"buckets,omitempty"`
			}{
				Enabled: true,
			},
		},
	}

	enabledProducts = config.EnabledProducts()

	expected = "OBJECT"
	if enabledProducts != expected {
		t.Errorf("Expected enabled products to be '%s', but got: %s", expected, enabledProducts)
	}
}

func TestJoinStrings(t *testing.T) {
	// Test case 1: Empty input slice
	result := joinStrings([]string{}, ",")
	if result != "" {
		t.Errorf("Expected empty string, but got: %s", result)
	}

	// Test case 2: Non-empty input slice with comma separator
	result = joinStrings([]string{"apple", "banana", "cherry"}, ",")
	expected := "apple,banana,cherry"
	if result != expected {
		t.Errorf("Expected '%s', but got: %s", expected, result)
	}

	// Test case 3: Non-empty input slice with space separator
	result = joinStrings([]string{"one", "two", "three"}, " ")
	expected = "one two three"
	if result != expected {
		t.Errorf("Expected '%s', but got: %s", expected, result)
	}
}
