package main

import (
	"bytes"
	"flag"
	"os"

	"github.com/prometheus/common/version"

	"net/http"

	"git.arvancloud.ir/project/ar-prometheus-exporter/collector"
	"git.arvancloud.ir/project/ar-prometheus-exporter/config"

	"github.com/prometheus/client_golang/prometheus"
	"github.com/prometheus/client_golang/prometheus/promhttp"
	log "github.com/sirupsen/logrus"
)

var (
	configFile  = flag.String("config", "/app/config.yml", "config file to load")
	logFormat   = flag.String("log-format", "text", "log format text or json (default json)")
	logLevel    = flag.String("log-level", "info", "log level")
	metricsPath = flag.String("path", "/metrics", "path to answer requests on")
	port        = flag.String("port", ":9786", "port number to listen on")

	cfg *config.Config
)

func init() {
	prometheus.MustRegister(version.NewCollector("arvancloud_exporter"))
}

func main() {
	flag.Parse()

	configureLog()

	log.Info("Welcome to Arvancloud Prometheus Exporter")

	log.Info("Version: 1.0.0")

	c, err := loadConfig()
	if err != nil {
		log.Errorf("Could not load config: %v", err)
		os.Exit(3)
	}
	cfg = c

	startServer()
}

func configureLog() {
	ll, err := log.ParseLevel(*logLevel)
	if err != nil {
		panic(err)
	}

	log.SetLevel(ll)

	if *logFormat == "text" {
		log.SetFormatter(&log.TextFormatter{
			ForceColors:      true,
			ForceQuote:       true,
			FullTimestamp:    true,
			DisableTimestamp: false,
			TimestampFormat:  "2006-01-02 15:04:05",
		})
	} else {
		log.SetFormatter(&log.JSONFormatter{
			TimestampFormat:  "2006-01-02 15:04:05",
			DisableTimestamp: false,
			DataKey:          "",
			PrettyPrint:      false,
		})
	}
}

func loadConfig() (*config.Config, error) {
	b, err := os.ReadFile(*configFile)
	if err != nil {
		return nil, err
	}

	return config.Load(bytes.NewReader(b))
}

func startServer() {
	h, err := createMetricsHandler()
	if err != nil {
		log.Fatal(err)
	}
	http.Handle(*metricsPath, h)

	http.HandleFunc("/health", func(w http.ResponseWriter, r *http.Request) {
		_, _ = w.Write([]byte("ok"))
	})

	http.HandleFunc("/", func(w http.ResponseWriter, r *http.Request) {
		_, _ = w.Write([]byte(`<html>
			<head><title>Arvancloud Exporter</title></head>
			<body>
			<h1>Arvancloud Exporter</h1>
			<p><a href="` + *metricsPath + `">Metrics</a></p>
			</body>
			</html>`))
	})

	log.Info("Start server on ", *port)
	log.Fatal(http.ListenAndServe(*port, nil))
}

func createMetricsHandler() (http.Handler, error) {
	opts := collectorOptions()
	nc, err := collector.NewCollector(cfg, opts...)
	if err != nil {
		return nil, err
	}

	registry := prometheus.NewRegistry()
	err = registry.Register(nc)
	if err != nil {
		return nil, err
	}

	return promhttp.HandlerFor(registry,
		promhttp.HandlerOpts{
			ErrorLog:      log.New(),
			ErrorHandling: promhttp.ContinueOnError,
		}), nil
}

func collectorOptions() []collector.Option {
	opts := []collector.Option{}

	if cfg.Products.CDN.Enabled {
		opts = append(opts, collector.WithCDN())
	}

	if cfg.Products.OBJECT.Enabled {
		opts = append(opts, collector.WithObject())
	}

	return opts
}
